<!DOCTYPE html>
<html lang="en">
<head>
<meta charset="utf-8" />
<meta content="width=device-width, initial-scale=1, shrink-to-fit=no" name="viewport" />
<title>{{ message }}</title>
</head>
<body>
<section class="card">
<div class="greeting [&_a]:flex">{{ message }}</div>
<button type="button">Close</button>
<button type="button" disabled>&times;</button>
</section>
</body>
</html>
