{#def title #}

<!DOCTYPE html>
<html lang="en">
<head>
<meta charset="utf-8" />
<meta content="width=device-width, initial-scale=1, shrink-to-fit=no" name="viewport" />
<title>{{ title }}</title>
</head>
<body>
{{ content }}
</body>
</html>
